/*
 * DRV887xEVM Firmware 1.3
 *
 * This firmware is loaded on the DRV887xEVM boards and allows users to
 * control the DRV887x device on the board using the potentiometers.
 *
 * Copyright (C) 2019 Texas Instruments Incorporated - http://www.ti.com/
 *
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *
 *    Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 *    Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the
 *    distribution.
 *
 *    Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 *  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 *  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 *  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ****************************************************************************
*  MSP430FR2100 / DRV887xEVM
*  SW REV 1.3
*
*  Description: Using DRV887x Customer EVM, potentiometers "IN1 PWM", and
*  "IN2 PWM" set the duty cycles for the PWM signals to the EN/IN1 and
*  PH/IN2 pins of the DRV887x. The PMODE jumper selects the mode for which the
*  DRV887x will interpret the input signals.
*
*  Board Function          MSP430FR2100
*  PWM output 1            P1.6 (TB0.1)
*  PWM output 2            P1.7 (TB0.2)
*  Speed Ref1 Pot          P1.0 (A0 in)
*  Speed Ref2 Pot          P1.1 (A1 in)
*  nSLEEP                  P2.1 (output)
*  Status LED              P2.7 (output)
******************************************************************************/

#include "msp430FR2100.h"

// Pins
#define SPD_REF_1   BIT0                 // P1.0
#define SPD_REF_2   BIT1                 // P1.1
#define PWM1        BIT6                 // P1.6
#define PWM2        BIT7                 // P1.7
#define nSLEEP      BIT1                 // P2.1
#define PMODE       BIT6                 // P2.6
#define STATUS_LED  BIT7                 // P2.7

int main(void)
{
  WDTCTL = WDTPW + WDTHOLD;              // Stop watchdog timer

  //FR2xxx, FR4xxx, FR58xx, FR59xx, FR6xxx devices
  // Configure one FRAM waitstate as required by the device datasheet for MCLK
  // operation beyond 8MHz _before_ configuring the clock system.
  FRCTL0 = FWPW | NWAITS_1; // Change the NACCESS_x value to add the right amount of waitstates, FWPW is the FRAM write password
  __delay_cycles(100);

  CSCTL1 = DCORSEL_5;                     // System Clock set to 16 MHz

  // Configure Pins
  P1OUT = 0x00;                          // P1.x Reset - outputs low
  P2OUT = 0x00;                          // P2.x reset - outputs low
  P1DIR = PWM1 + PWM2;                   // PWM outputs
  P2DIR = STATUS_LED + nSLEEP;           // Status LED and nSLEEP output
  P1REN = BIT3+BIT4+BIT5;                // Enable pulldown resistors

  // Disable the GPIO power-on default high-impedance mode to activate
  // previously configured port settings
  PM5CTL0 &= ~LOCKLPM5;

  // Configure ADC
  ADCCTL0 |= ADCSHT_2 | ADCON;                       // 16ADCclks, ADC ON
  ADCCTL1 |= ADCSHP | ADCCONSEQ_0 | ADCSSEL_1;       // ADC clock ACLK, sampling timer, s/w trig.,single sequence
  ADCCTL2 &= ~ADCRES_1;                              // 8-bit conversion results

  P2OUT |=  STATUS_LED + nSLEEP;                     // STATUS LED on

  __delay_cycles(10);

  // Determine PMODE state
  char pmode_state = 0;
  char adc_ch1_results;
  P2OUT |= PMODE;               // input pullup
  P2REN |= PMODE;               // enable input resistor
  pmode_state += P2IN & PMODE;  // store state
  P2OUT &= ~PMODE;              // input pulldown
  pmode_state += P2IN & PMODE;  // store state
  P2REN &= ~PMODE;              // disable input resistor

  // Configure Timer Peripheral for PWM
  TB0CCR0 = 115;                            // PWM Period
  TB0CCTL1 = OUTMOD_7;                      // CCR1 reset/set
  TB0CCR1 = 0;                             // CCR1 PWM duty cycle
  P1SEL0 |= SPD_REF_1 + SPD_REF_2;

  if(pmode_state == 0)
  {
      P1OUT &= ~PWM2;
      P1SEL1 |= SPD_REF_1 + SPD_REF_2 + PWM1;
  }
  else
  {
      P1SEL1 |= SPD_REF_1 + SPD_REF_2 + PWM1 + PWM2;

      TB0CCTL2 = OUTMOD_7;                      // CCR2 reset/set
      TB0CCR2 = 0;                              // CCR2 PWM duty cycle
  }

  TB0CTL = TBSSEL__SMCLK | MC__UP | TBCLR | ID_1;  // SMCLK, up mode, clear TBR, Divide clk by 2
//  TB0CTL = TBSSEL__SMCLK | MC__UP | TBCLR;  // SMCLK, up mode, clear TBR

  while(1)
  {
      ADCCTL0 &= ~(ADCENC | ADCSC);
      ADCMCTL0 = ADCINCH_1 + ADCSREF_0;       // A1; Vref=AVCC
      ADCCTL0 |= ADCENC | ADCSC;              // Sampling and conversion start
      while(ADCCTL1 & ADCBUSY);               // Wait for conversion complete
      adc_ch1_results = ADCMEM0;              // Put ADC value into variable before loading into TB0CCR2

      if(pmode_state == 0)
      {
          if(adc_ch1_results <= 53) P1OUT &= ~PWM2;
          else if(adc_ch1_results >= 73) P1OUT |= PWM2;
      }
      else
          TB0CCR2 = adc_ch1_results;

      ADCCTL0 &= ~(ADCENC | ADCSC);
      ADCMCTL0 = ADCINCH_0 + ADCSREF_0;       // A0; Vref=AVCC
      ADCCTL0 |= ADCENC | ADCSC;              // Sampling and conversion start
      while(ADCCTL1 & ADCBUSY);               // Wait for conversion complete
      TB0CCR1 = ADCMEM0;                      // Put ADC value into TB0CCR1
  }
}
